<?xml version="1.0"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0">
<xsl:output method="text"/>
<xsl:strip-space elements="*"/>

<xsl:param name="timestamp"/>
<xsl:param name="user"/>

<xsl:template match="firewall">
###
### Generated by PfPro	( http://pfpro.bandwidth-junkies.net/ )
### by <xsl:value-of select="$user"/> at <xsl:value-of select="$timestamp"/>
###

########################################################################
# BEGIN MACROS
########################################################################
<xsl:apply-templates select="macros"/>
########################################################################
# BEGIN NORMALIZATION
########################################################################
<xsl:apply-templates select="normalization"/>
########################################################################
# BEGIN FILTERING
########################################################################
<xsl:apply-templates select="filtering"/>
</xsl:template>

<xsl:template match="macros">
	<xsl:for-each select="macro">
		<xsl:value-of select="@name"/>
		<xsl:text>="</xsl:text>
		<xsl:value-of select="@value"/>
		<xsl:text>"&#xa;</xsl:text>
	</xsl:for-each>
</xsl:template>

<xsl:template match="normalization">
	<xsl:for-each select="scrub">
		<xsl:text>scrub </xsl:text>
		<xsl:value-of select="@direction"/>
		<xsl:text> all</xsl:text>

		<xsl:if test="no-df">
			<xsl:text> no-df</xsl:text>
		</xsl:if>

		<xsl:if test="max-mss">
			<xsl:text> max-mss </xsl:text>
			<xsl:value-of select="max-mss/@value"/>
		</xsl:if>

		<xsl:if test="random-id">
			<xsl:text> random-id</xsl:text>
		</xsl:if>

		<xsl:text>&#xa;</xsl:text>
	</xsl:for-each>
</xsl:template>

<xsl:template match="filtering">
	<xsl:for-each select="rule">
		<!-- HANDLE USER-SPECIFIED COMMENT -->
		<xsl:if test="comment">
			<xsl:apply-templates select="comment"/>
		</xsl:if>

		<!-- PASS/BLOCK IN/OUT -->
		<xsl:value-of select="@option"/>
		<xsl:text> </xsl:text>
		<xsl:value-of select="@direction"/>

		<!-- LOGGING -->
		<xsl:if test="@log = 'true'">
			<xsl:text> log</xsl:text>
		</xsl:if>

		<!-- APPLY RULE QUICKLY? -->
		<xsl:if test="@quick = 'true'">
			<xsl:text> quick</xsl:text>
		</xsl:if>

		<!-- HANDLE INTERFACE SPECIFICATION -->
		<xsl:if test="@interface">
			<xsl:text> on </xsl:text>
			<xsl:value-of select="@interface"/>
		</xsl:if>

		<!-- INET OR INET6 -->
		<xsl:if test="ip">
			<xsl:text> </xsl:text><xsl:value-of select="@ip"/>
		</xsl:if>

		<!-- HANDLE PROTOCOLS -->
		<xsl:if test="protocols/proto">
			<xsl:apply-templates select="protocols"/>
		</xsl:if>

		<!-- HANDLE TO AND FROM -->
		<xsl:apply-templates select="from"/>
		<xsl:apply-templates select="to"/>

		<xsl:if test="all">
			<xsl:text> all</xsl:text>
		</xsl:if>

		<!-- HANDLE PORTS -->
		<xsl:if test="ports/port">
			<xsl:apply-templates select="ports"/>
		</xsl:if>

		<xsl:if test="users/user">
			<xsl:apply-templates select="users"/>
		</xsl:if>

		<xsl:if test="@state != ''">
			<xsl:text> </xsl:text>
			<xsl:value-of select="@state"/>
			<xsl:text> state</xsl:text>
		</xsl:if>

		<xsl:text>&#xa;</xsl:text>
	</xsl:for-each>
</xsl:template>

<xsl:template match="comment">
	<xsl:text>### </xsl:text>
	<xsl:value-of select="."/>
	<xsl:text>&#xa;</xsl:text>
</xsl:template>

<xsl:template match="protocols">
	<xsl:text> proto </xsl:text>

	<xsl:choose>
		<xsl:when test="count(proto) &gt; 1">
			<xsl:text>{ </xsl:text>

			<xsl:for-each select="proto">
				<xsl:value-of select="@type"/>

				<xsl:if test="position() != last()">
					<xsl:text>, </xsl:text>
				</xsl:if>
			</xsl:for-each>

			<xsl:text> }</xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="proto/@type"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="from">
	<xsl:text> from </xsl:text>
	<xsl:value-of select="@address"/>
</xsl:template>

<xsl:template match="to">
	<xsl:text> to </xsl:text>
	<xsl:value-of select="@address"/>
</xsl:template>

<xsl:template match="ports">
	<xsl:text> port </xsl:text>

	<xsl:choose>
		<xsl:when test="count(port) &gt; 1">
			<xsl:text>{ </xsl:text>

			<xsl:for-each select="port">
				<xsl:value-of select="@number"/>

				<xsl:if test="position() != last()">
					<xsl:text>, </xsl:text>
				</xsl:if>
			</xsl:for-each>

			<xsl:text> }</xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="port/@number"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

<xsl:template match="users">
	<xsl:text> user </xsl:text>

	<xsl:choose>
		<xsl:when test="count(user) &gt; 1">
			<xsl:text>{ </xsl:text>

			<xsl:for-each select="user">
				<xsl:value-of select="@name"/>

				<xsl:if test="position() != last()">
					<xsl:text>, </xsl:text>
				</xsl:if>
			</xsl:for-each>

			<xsl:text> }</xsl:text>
		</xsl:when>
		<xsl:otherwise>
			<xsl:value-of select="user/@name"/>
		</xsl:otherwise>
	</xsl:choose>
</xsl:template>

</xsl:stylesheet>
